@@ -56,7 +56,7 @@ function() {
    # choose k
    cat("---------\n")
    continue <- F
-   this.text <- paste("Please enter the number of stochastic profiling samples you wish to generate:\n(default: ",k.default,")\n",sep="")
+   this.text <- paste("Please enter the number of stochastic profiling observations you wish to generate:\n(default: ",k.default,")\n",sep="")
    while (!continue) {
       k <- readline(this.text)
       if (k=="") { k <- k.default }
@@ -77,22 +77,296 @@ function() {
    # choose n
    cat("---------\n")
    continue <- F
-   this.text <- paste("Please enter the number of cells that should enter each sample:\n(default: ",n.default,")\n",sep="")
+   this.text <- "Next we enter the number of cells that should enter each observation, which case do you want:\n 1: all observations should contain the same number of cells, or \n 2: each observation contains a different number of cells \n(default: 1).\n"
    while (!continue) {
-      n <- readline(this.text)
-      if (n=="") { n <- n.default }
-      if (is.na(as.numeric(n))) {
-         this.text <- "Invalid choice. Please enter a finite natural number.\n"
-      }
-      else {
-         n <- as.numeric(n)
-         if ((round(n)==n) && ((n>0) && (n<Inf))) {
-            continue <- T
-         }
-         else {
-            this.text <- "Invalid choice. Please enter a finite natural number.\n"
-         }
-      }
+       number <- readline(this.text)
+       if (number=="") {
+           number <- 1
+           continue = T
+       }
+       if (number %in% 1:2) {
+           continue = T
+       }
+       else {
+           this.text <- "Invalid choice. Please try again.\n"
+       }
+   }
+
+   if(number == 2){
+   cat("---------\n")
+   continue <- F
+   this.text <- "How would you like to input the number of cells for each observation?\n 1: enter manually\n 2: read from file\n 3: enter the name of a variable\n(default: 1).\n"
+   while (!continue) {
+       input_n <- readline(this.text)
+       if (input_n=="") {
+           input_n <- 1
+       }
+       if (input_n %in% 1:3) {
+           continue <- T
+       }
+       else {
+           this.text <- "Invalid choice. Please try again.\n"
+       }
+   }
+   if(input_n == 1){
+       cat("---------\n")
+       continue <- F
+       this.text <- paste("Please enter the number of cells that should enter each observation:\nseparated by either commas or spaces,\ne.g. in case of five observations\n5, 10, 2, 10, 5\n or\n5 10 2 10 5.\n(default: ",n.default,")\n",sep="")
+
+       while (!continue) {
+           n <- readline(this.text)
+           if (n=="") { n <- n.default }
+
+           else{
+               # try comma separation
+               n.tmp <- suppressWarnings(as.numeric(unlist(strsplit(n, ","))))
+               # try space separation
+               if (any(is.na(n.tmp))) {
+                   n <- suppressWarnings(as.numeric(unlist(strsplit(n, " "))))
+               }
+               else {
+                   n <- n.tmp
+               }
+               if (any(is.na(n)) || (round(n)!=n)) {
+                   this.text <- "Invalid choice. Please enter only finite natural numbers. Please try again.\n"
+               }
+               else {
+                   if (any(abs(n)==Inf)) {
+                       this.text <- "There are a infinite values. Please enter only finite natural numbers.\n"
+                   }
+                   else if (any(n<=0)) {
+                       this.text <- "There are non-positive values. Please enter only finite natural numbers.\n"
+                   }
+                   else {
+                       if (length(n) != 1 && length(n)!=k) {
+                           this.text <- "The length of n does not agree with the number of\nobservations in the data. Please try again.\n"
+                       }
+                       else {
+                           continue <- T
+                       }
+                   }
+               }
+           }
+       }
+
+
+   } else if (input_n == 2){
+       # choose filename
+       cat("---------\n")
+       continue_alg <- F
+
+       while(!continue_alg){
+       continue <- F
+       cat("The file should contain a data matrix with a one dimensional vector standing for the observations. Fields have to be separated by tabs or white\nspaces, but not by commas. If necessary, please delete the commas in the\ntext file using the \'replace all\' function of your text editor.\n")
+
+       cat("\nPlease enter a valid path and filename, either a full path, e.g.\n",getwd(),"/mydata.txt\nor just a file name, e.g.\nmydata.txt.\nThe current directory is\n",getwd(),".\n",sep="")
+       this.text <- ""
+
+
+       while (!continue) {
+           path <- readline(this.text)
+
+           if (!file.exists(path)) {
+               this.text <- "This file does not exist. Please try again.\n"
+           }
+           else {
+               continue <- T
+           }
+       }
+
+       continue <- F
+       this.text <- "Does the file contains a column name? Please enter 'yes' or 'no'.\n"
+       while (!continue) {
+           cnames <- readline(this.text)
+           if ((cnames=="") || (!(cnames %in% c("yes","no")))) {
+               this.text <- "Please enter 'yes' or 'no'.\n"
+           }
+           else {
+               continue <- T
+           }
+       }
+
+       continue <- F
+       this.text <- "Does the file contain row names? Please enter 'yes' or 'no'.\n"
+       while (!continue) {
+           rnames <- readline(this.text)
+           if ((rnames=="") || (!(rnames %in% c("yes","no")))) {
+               this.text <- "Please enter 'yes' or 'no'.\n"
+           }
+           else {
+               continue <- T
+           }
+       }
+
+       continue <- F
+       this.text <- "Do the rows or the columns stand for the different observations?\n 1: rows\n 2: columns.\n"
+       while (!continue) {
+           dimn <- readline(this.text)
+           if ((dimn=="") || (!(dimn %in% 1:2))) {
+               this.text <- "Invalid choice. Please try again.\n"
+           }
+           else {
+               continue <- T
+           }
+       }
+
+       # read file
+
+       if (rnames=="yes") {
+          n <- read.table(file=path,header=(cnames=="yes"),row.names=1)
+       }
+       else {
+           n <- read.table(file=path,header=(cnames=="yes"))
+       }
+       if (dimn==2) {
+           n <- t(n)
+       }
+
+
+       if (any(is.na(n)) || any(round(n)!=n)) {
+           cat("Invalid choice. Please enter finite natural numbers. Please try again.\n\n\n")
+       } else {
+           if (any(abs(n)==Inf)) {
+               cat("There are infinite values. Please enter only finite natural numbers.\n\n\n")
+           }
+           else if (any(n<=0)) {
+               cat("There are non-positive values. Please enter only finite natural numbers.\n\n\n")
+               }
+           else {
+               if (length(n) != 1 && length(n)!=k) {
+                   cat("The number of observations does not agree with the number of\nobservations of the data. Please try again.\n\n\n")
+                }
+               else {
+                   continue_alg <- T
+               }
+           }
+       }
+       }
+
+
+
+       } else if(input_n == 3){
+       cat("---------\n")
+       continue_alg <- F
+
+       while(!continue_alg){
+       continue <- F
+
+       this.text <- "The variable should be a matrix with one dimension standing for the observations. Please enter the name of the variable.\n"
+       while (!continue) {
+           n_variablename <- readline(this.text)
+           n_variable <- try(eval(parse(text=n_variablename)),silent=T)
+
+           if (class(n_variable)=="try-error") {
+               this.text <- "This variable does not exist. Please try again.\n"
+           }
+           else {
+               continue <- T
+               if (is.data.frame(n_variable)) {
+                   cat("Data frame has been converted to matrix.\n")
+                   n_variable <- as.matrix(n_variable)
+               }
+               if (!is.matrix(n_variable)) {
+                   cat(paste("This is not a matrix. The variable is converted to a matrix with 1 column\nand ",length(n_variable)," rows.\n\n",sep=""))
+                   n_variable <- matrix(n_variable,ncol=1)
+               }
+           }
+       }
+
+       continue <- F
+       this.text <- "Do the rows or the columns stand for the different observations?\n 1: rows\n 2: columns.\n"
+       while (!continue) {
+           dimn <- readline(this.text)
+           if ((dimn=="") || (!(dimn %in% 1:2))) {
+               this.text <- "Invalid choice. Please try again.\n"
+           }
+           else {
+               continue <- T
+               if (dimn==1) {
+                   n <- n_variable
+               }
+               else {
+                   n<- t(n_variable)
+               }
+           }
+       }
+       if (is.null(colnames(n))) {
+           n_cnames <- "no"
+       }
+       else {
+           n_cnames <- "yes"
+       }
+       n_rnames <- "no"
+
+
+       if (any(is.na(n)) || any(round(n)!=n)) {
+           cat("Invalid choice. Please enter finite natural numbers. Please try again.\n\n\n")
+       } else {
+           if (any(abs(n)==Inf)) {
+               cat("There are infinite values. Please enter only finite natural numbers.\n\n\n")
+           }
+           else if (any(n<=0)) {
+               cat("There are non-positive values. Please enter only finite natural numbers.\n\n\n")
+           }
+           else {
+               if (length(n) != 1 && length(n)!=k) {
+                   cat("The number of observations does not agree with the number of\nobservations of the data. Please try again.\n\n\n")
+               }
+               else {
+                   continue_alg <- T
+               }
+           }
+       }
+
+
+       }
+
+       }
+   }
+
+
+   if(number == 1){
+   cat("---------\n")
+   continue <- F
+   this.text <- paste("Please enter the number of cells that should enter each sample:\n(default: ",n.default,")\n",sep="")
+
+   while (!continue) {
+       n <- readline(this.text)
+       if (n=="") { n <- n.default }
+
+       else{
+           # try comma separation
+           n.tmp <- suppressWarnings(as.numeric(unlist(strsplit(n, ","))))
+           # try space separation
+           if (any(is.na(n.tmp))) {
+               n <- suppressWarnings(as.numeric(unlist(strsplit(n, " "))))
+           }
+           else {
+               n <- n.tmp
+           }
+           if (any(is.na(n))) {
+               this.text <- "Invalid choice. Please enter a finite natural number. Please try again.\n"
+           }
+           else {
+               if (any(abs(n)==Inf)) {
+                   this.text <- "There is a infinite value. Please enter a finite natural number.\n"
+               }
+               else if (any(n<=0)) {
+                   this.text <- "There is a non-positive value. Please enter a finite natural number.\n"
+               }
+               else {
+                   if (length(n) != 1) {
+                       this.text <- "Invalid choice. Please enter a finite natural number. Please try again.\n"
+                   }
+                   else {
+                       continue <- T
+                   }
+               }
+           }
+
+       }
+
+   }
    }
 
    # choose m
@@ -419,42 +693,76 @@ function() {
    par(ask=T)
 
    # Prompting finished!! Now generate data.
+
+
+   if(length(n) == 1){
+      N.matrix <- t(rmultinom(n=k, size=n, prob=p))
+      } else {
+      N.matrix <- matrix(0, nrow=k, ncol=length(p))
+      for(j in 1:k){
+         N.matrix[j,] <- t(rmultinom(n=1, size=n[j], prob=p))
+      }
+   }
+
+
    dataset <- matrix(NA,nrow=k,ncol=m)
    for (g in 1:m) {
        if (model==1) {
           set.model.functions("LN-LN")
-          dataset[,g] <- r.sum.of.mixtures.LNLN(k,n,p,mu.list[[g]],rep(sigma,TY))
+          dataset[,g] <- r.sum.of.mixtures.LNLN(k,n,p,mu.list[[g]],rep(sigma,TY), N.matrix)
           x <- seq(round(min(dataset[,g])),round(max(dataset[,g])),(round(max(dataset[,g]))-round(min(dataset[,g])))/500)
+          if(number == 1){
           y <- d.sum.of.mixtures.LNLN(x,n,p,mu.list[[g]],rep(sigma,TY),logdens=F)
           xlabel <- "Sum of mixtures of lognormals"
+          }
+          else{
+          y <- mix.d.sum.of.mixtures.LNLN(x, n, p,mu.list[[g]], rep(sigma,TY), logdens=F)
+          xlabel <- "Mixed density of mixtures of lognormals"
+          }
        }
        else if (model==2) {
           set.model.functions("rLN-LN")
-          dataset[,g] <- r.sum.of.mixtures.rLNLN(k,n,p,mu.list[[g]],sigma)
+          dataset[,g] <- r.sum.of.mixtures.rLNLN(k,n,p,mu.list[[g]],sigma, N.matrix)
           x <- seq(round(min(dataset[,g])),round(max(dataset[,g])),(round(max(dataset[,g]))-round(min(dataset[,g])))/500)
-          y <- d.sum.of.mixtures.rLNLN(x,n,p,mu.list[[g]],sigma,logdens=F)
-          xlabel <- "Sum of mixtures of lognormals"
-       }
-       else if (model==3) {
+          if(number == 1){
+              y <- d.sum.of.mixtures.rLNLN(x,n,p,mu.list[[g]],sigma,logdens=F)
+              xlabel <- "Sum of mixtures of lognormals"
+          }
+          else{
+              y <- mix.d.sum.of.mixtures.rLNLN(x, n, p,mu.list[[g]], sigma, logdens=F)
+              xlabel <- "Mixed density of mixtures of lognormals"
+          }
+        }
+        else if (model==3) {
           set.model.functions("EXP-LN")
-          dataset[,g] <- r.sum.of.mixtures.EXPLN(k,n,p,mu.list[[g]],sigma,lambda.list[[g]])
+          dataset[,g] <- r.sum.of.mixtures.EXPLN(k,n,p,mu.list[[g]],sigma,lambda.list[[g]], N.matrix)
           x <- seq(round(min(dataset[,g])),round(max(dataset[,g])),(round(max(dataset[,g]))-round(min(dataset[,g])))/500)
-          y <- d.sum.of.mixtures.EXPLN(x,n,p,mu.list[[g]],sigma,lambda.list[[g]],logdens=F)
-          xlabel <- "Sum of mixtures of lognormals and exponentials"
+          if(number == 1){
+              y <- d.sum.of.mixtures.EXPLN(x,n,p,mu.list[[g]],sigma,lambda.list[[g]],logdens=F)
+              xlabel <- "Sum of mixtures of lognormals and exponentials"
+          }
+          else{
+              y <- mix.d.sum.of.mixtures.EXPLN(x, n, p,mu.list[[g]], sigma,lambda.list[[g]], logdens=F)
+              xlabel <- "Mixed density of mixtures of lognormals and exponentials"
+          }
        }
        hist(dataset[,g],main=paste("Gene",g),breaks=50,xlab=xlabel,ylab="Density",freq=F,col="lightgrey")
        lines(x,y,col="blue",lwd=3)
+       if (number == 1){
        legend("topright",legend=c("data generating pdf"),col=c("blue"),lty=1,lwd=3)
-
+       }
+       else{
+           legend("topright",legend=c("data generating mixed pdf"),col=c("blue"),lty=1,lwd=3)
+       }
    }
    colnames(dataset) <- paste("gene",1:m)
-   rownames(dataset) <- paste("sample",1:k)
+   rownames(dataset) <- paste("observation",1:k)
 
    par(ask=F)
 
    # Print result.
    if (k>50) {
-      cat("\n\nThe dataset has been generated. The first 50 samples are:\n\n")
+      cat("\n\nThe dataset has been generated. The first 50 observations are:\n\n")
    }
    else {
       cat("\n\nThe dataset has been generated:\n\n")
